/*
 * Copyright (c) 2017-2022, Lawrence Livermore National Security, LLC and
 * other Axom Project Developers. See the top-level LICENSE file for details.
 *
 * SPDX-License-Identifier: (BSD-3-Clause)
 */

#ifndef AXOM_COMMON_CONFIG_HPP
#define AXOM_COMMON_CONFIG_HPP

/*
 * Note: Use only C-style comments in this file
 *       since it might be included from a C file
 */


/*
 * Axom Version Information
 */
#define AXOM_VERSION_MAJOR 0
#define AXOM_VERSION_MINOR 7
#define AXOM_VERSION_PATCH 0
#define AXOM_VERSION_FULL  "v0.7.0"
/*NOTE: Do not add AXOM_GIT_SHA here.  This will force
 * full rebuild on all local changes. It is added in axom::getVersion()
 * and axom::about(), as well as accessible from axom::gitSha().
 */


/*
 * Axom source location
 */
#define AXOM_SRC_DIR "/wrkdirs/usr/ports/science/axom/work/axom-0.7.0/src"
#define AXOM_BIN_DIR "/wrkdirs/usr/ports/science/axom/work/.build"
#define AXOM_DATA_DIR "/wrkdirs/usr/ports/science/axom/work/axom-0.7.0/data"

/*
 * Indicates whether Axom is compiled with Annotations that can facilitate
 * evaluation of performance with various performance tools.
 */
/* #undef AXOM_USE_ANNOTATIONS */

/*
 * Platform specific definitions
 */
#define AXOM_CXX_STD "c++14"
/* #undef AXOM_CUDA_ARCH */
/* #undef AXOM_NO_INT64_T */
/* #undef AXOM_USE_MPI3 */

#ifndef AXOM_NO_INT64_T
/* #undef AXOM_USE_64BIT_INDEXTYPE */
#endif


/*
 * Compiler tests
 */
/* #undef USE_C_LOC_WITH_ASSUMED_SHAPE */


/*
 * Compiler defines for libraries (built-in and third party)
 */
/* #undef AXOM_USE_C2C */
#define AXOM_USE_CLI11
#define AXOM_USE_CONDUIT
/* #undef AXOM_USE_CUDA */
/* #undef AXOM_USE_HIP */
#define AXOM_USE_FMT
#define AXOM_USE_HDF5
#define AXOM_USE_LUA
#define AXOM_USE_MFEM
#define AXOM_USE_MPI
/* #undef AXOM_USE_MPIF_HEADER */
/* #undef AXOM_USE_OPENMP */
/* #undef AXOM_USE_RAJA */
/* #undef AXOM_USE_SCR */
#define AXOM_USE_SOL
#define AXOM_USE_SPARSEHASH
/* #undef AXOM_USE_UMPIRE */


/*
 * We need to explicitly add some definitions on Windows
 * for math defines (_USE_MATH_DEFINES), and to disambiguate
 * Windows GDI geometry classes (NOGDI).
 */
#ifdef WIN32
#define _USE_MATH_DEFINES
#define NOGDI
#define NOMINMAX
#endif


/*
 * Compiler defines for Axom components
 */
#define AXOM_USE_INLET
#define AXOM_USE_KLEE
#define AXOM_USE_MINT
#define AXOM_USE_LUMBERJACK
#define AXOM_USE_PRIMAL
#define AXOM_USE_QUEST
#define AXOM_USE_SIDRE
#define AXOM_USE_SLAM
#define AXOM_USE_SLIC
#define AXOM_USE_SPIN


/*
 * Inline axom's macros related to export symbols, e.g. AXOM_EXPORT
 */

#ifndef AXOM_EXPORT_H
#define AXOM_EXPORT_H

#ifdef AXOM_STATIC_DEFINE
#  define AXOM_EXPORT
#  define AXOM_NO_EXPORT
#else
#  ifndef AXOM_EXPORT
#    ifdef axom_EXPORTS
        /* We are building this library */
#      define AXOM_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define AXOM_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef AXOM_NO_EXPORT
#    define AXOM_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef AXOM_DEPRECATED
#  define AXOM_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef AXOM_DEPRECATED_EXPORT
#  define AXOM_DEPRECATED_EXPORT AXOM_EXPORT AXOM_DEPRECATED
#endif

#ifndef AXOM_DEPRECATED_NO_EXPORT
#  define AXOM_DEPRECATED_NO_EXPORT AXOM_NO_EXPORT AXOM_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef AXOM_NO_DEPRECATED
#    define AXOM_NO_DEPRECATED
#  endif
#endif

#endif /* AXOM_EXPORT_H */



/*
 * Compiler defines to configure the built-in fmt library
 * Redefines AXOM_FMT_DEPRECATED to workaround nvcc compiler error
 */
#define AXOM_FMT_EXCEPTIONS 0
#define AXOM_FMT_HEADER_ONLY 1
#ifndef WIN32
#define AXOM_FMT_DEPRECATED AXOM_DEPRECATED
#endif

/*
 * Compiler defines to configure the built-in sparsehash library
 */
#define SPARSEHASH_HASHFUN_HEADER <functional>
#define SPARSEHASH_HASHFUN_NAMESPACE std


/*
 * Disable some MSVC warnings related to shared libraries, if applicable
 */



#endif  /* AXOM_COMMON_CONFIG_HPP */
